#include <SPI.h>
#include <Streaming.h>  // http://arduiniana.org/libraries/streaming
#include <ff.h>
#include <sdcard.h>

FATFS FatFs;      // Work area (file system object) for logical drive
SdFatFs sdFatFs;  // FatFs class
SdSpi sdSpi;      // SD SPI class

FRESULT writeFile (char*, char*, uint16_t);
FRESULT readFile (char*);
void showError(FRESULT fr, char*);

FRESULT fr;             // FatFs return code
const uint8_t readBufLen = 128;  // length inBuffer

void setup() {
  Serial.begin(115200);
  Serial << "\n\n";
  sdSpi.pinSS((byte)D4);  // SD Select pin
  char fName[13], str[readBufLen + 1];

//  if ( sdFatFs.f_mount(&FatFs, "", 0) != FR_OK) showError(fr, "SD\0");
  fr = sdFatFs.f_mount(&FatFs, "", 0);  // init SD or test if memory card is inserted
  if (fr != FR_OK) showError(fr, '\0');
  else {
    Serial << (F("\r\n#### Read text from READ.TXT ####\n"));
    strcpy(fName, "read.txt");
    fr = readFile(fName);  // read a text file
    if (fr != FR_OK) showError(fr, fName);

    Serial << (F("\r\n#### Write/Add text to WRITE.TXT ####\n"));
    strcpy(fName, "write.txt");
    strcpy(str, "This text is added to the file.\n\0");
    fr = writeFile(fName, str, strlen(str));  // write to text file
    if (fr != FR_OK) showError(fr, fName);

    Serial << (F("\r\n#### Read added text back ####\n"));
    fr = readFile(fName);  // read the text back
    if (fr != FR_OK) showError(fr, fName);
  }
}

void loop() { }

FRESULT readFile (char* fName) {  // Read and display the contents of a file
  FIL fil;                // File object
  char line[readBufLen];  // Line buffer
  fr = sdFatFs.f_open(&fil, fName, FA_READ);  // open file
  if (fr != FR_OK) return fr;
  while (sdFatFs.f_gets(line, readBufLen, &fil)) Serial << line;  // read lines until a '\n' en print it
  Serial << endl;
  return sdFatFs.f_close(&fil);  // Close file
}

FRESULT writeFile (char* fName, char* fTxt, uint16_t bytes) {  // Open a file as new/append and write the text
  FIL fil;     // File object
  uint16_t b;  // bytes written
  //fr = sdFatFs.f_open(&fil, fName, FA_CREATE_ALWAYS);  // Creates a new file. If the file is existing, it will be truncated and overwritten.
  fr = sdFatFs.f_open(&fil, fName, FA_OPEN_ALWAYS);  // Opens the file if it is existing. If not, a new file is created.
  if (fr != FR_OK) return fr;
  fr = sdFatFs.f_lseek(&fil, f_size(&fil));  // Move read/write pointer to end of file to append text
  if (fr != FR_OK) return fr;
  fr = sdFatFs.f_write(&fil, fTxt, bytes, &b);   // append text to file
  if (fr != FR_OK) return fr;
  return sdFatFs.f_close(&fil);  // Close file
}

void showError(FRESULT fr, char* in) {  // display error
  const char errors[][66] = {  // demo errors (fr: enum FRESULT{..} in ff.h)
    "Succeeded",
    "A hard error occurred in the low level disk I/O layer.",
    "Assertion failed.",
    "The physical drive cannot work.",
    "Could not find the file.",
    "Could not find the path.",
    "The path name format is invalid.",
    "Access denied due to prohibited access or directory full.",
    "Access denied due to prohibited access.",
    "The file/directory object is invalid.",
    "The physical drive is write protected.",
    "The logical drive number is invalid.",
    "The volume has no work area.",
    "There is no valid FAT volume.",
    "The f_mkfs() aborted due to any parameter error.",
    "Could not get a grant to access the volume within defined period.",
    "The operation is rejected according to the file sharing policy.",
    "LFN working buffer could not be allocated.",
    "Number of open files > _FS_SHARE.",
    "Given parameter is invalid."
  };
  Serial << "\tERROR (" << in << "): " << errors[fr] << endl;
}