/***************************************************
 *        (C)ChaN 2014 FatFs (Fat Filesysten)      *
 *        Stripped and modified for ESP8266        *
 *             by Ewald Kendziorra sr              *
 *          2014-2016, www.kendziorra.nl           *
 *  Use it with my Arduino NodeMCU v2 + SD Shield  *
 * FatFs uses 8.946 bytes of program storage space *
 * FatFs uses 820 bytes of dynamic memory          *
 *  DIR functions are stripped, only ROOT is used  *
 ***************************************************/

// Explanation FatFs funtions see below
// ------------------------------------

// SdFatFsESP.ino
// ==============
/*-------------------------------------------------------------------------------------/
 /  Demo FatFs file for my Arduino NodeMCU v2 + SD shield
 /-------------------------------------------------------------------------------------/
 /  Installed CODE PAGE (see below) is 1252, Latin 1 (Windows) (in ff.cpp)
 /      other code pages in file ff.c in download from http://elm-chan.org/fsw/ff/ff10a.zip
 /------------------------------------------------------------------------------------*/

// sdcard.h/cpp
// ============
/*--------------------------------------------------------------------------------------------/
 /  Low level SD card routines used by FatFs (C)ChaN, 2013
 /--------------------------------------------------------------------------------------------/
 /  Modified and ported to software SPI for Arduino NodeMCU v2 + SD shield
 /  by Ewald Kendziorra sr, 2014, www.kendziorra.nl
 /-------------------------------------------------------------------------------------------*/

// ff.h/cpp
// ========
/*--------------------------------------------------------------------------------------------/
 /  FatFs - FAT file system module include file  R0.10a  (C)ChaN, 2014
 /--------------------------------------------------------------------------------------------/
 /  Modified for Arduino NodeMCU v2 + SD shield 
 /  by Ewald Kendziorra sr, 2014, www.kendziorra.nl
 /--------------------------------------------------------------------------------------------/
 / FatFs module is a generic FAT file system module for small embedded systems.
 / This is a free software that opened for education, research and commercial
 / developments under license policy of following terms.
 /
 /  Copyright (C) 2014, ChaN, all right reserved.
 /
 / * The FatFs module is a free software and there is NO WARRANTY.
 / * No restriction on use. You can use, modify and redistribute it for
 /  personal, non-profit or commercial product UNDER YOUR RESPONSIBILITY.
 / * Redistributions of source code must retain the above copyright notice.
 /
 /--------------------------------------------------------------------------------------------/
 / ##################################################################
 / ####### many ChaN FatFs universal definitions are stripped #######
 / ##################################################################
 /
 / ################################################
 / ####### put your own code page in ff.cpp #######
 / ################################################
 / Installed CODE PAGE is 1252, Latin 1 (Windows) (in ff.cpp)
 /      other code pages in file ff.c http://elm-chan.org/fsw/ff/ff10a.zip
 / The _CODE_PAGE specifies the OEM code page to be used on the target system.
 / Incorrect setting of the code page can cause a file open failure.
 /--------------------------------------------------------------------------------------------/
 / * SdFat FatFs module provides following functions to the applications:
 /   ====================================================================
 / f_mount  - Register/Unregister a work area            http://elm-chan.org/fsw/ff/en/mount.html
 / f_open   - Open/Create a file-                        http://elm-chan.org/fsw/ff/en/open.html
 /              FA_READ, FA_WRITE, FA_OPEN_EXISTING,FA_OPEN_ALWAYS
 /              FA_CREATE_NEW, FA_CREATE_ALWAYS
 / f_close  - Close an open file                         http://elm-chan.org/fsw/ff/en/close.html
 / f_read   - Read file                                  http://elm-chan.org/fsw/ff/en/read.html
 / f_write  - Write file                                 http://elm-chan.org/fsw/ff/en/write.html
 / f_lseek  - Move read/write pointer, Expand file size  http://elm-chan.org/fsw/ff/en/lseek.html
 / f_sync   - Flush cached data                          http://elm-chan.org/fsw/ff/en/sync.html
 / f_gets   - Read a string                              http://elm-chan.org/fsw/ff/en/gets.html
 / f_putc   - Write a character                          http://elm-chan.org/fsw/ff/en/putc.html
 / f_puts   - Write a string                             http://elm-chan.org/fsw/ff/en/puts.html
 / f_tell   - Get current read/write pointer             http://elm-chan.org/fsw/ff/en/tell.html
 / f_eof    - Test for end-of-file on a file             http://elm-chan.org/fsw/ff/en/eof.html
 / f_size   - Get size of a file                         http://elm-chan.org/fsw/ff/en/size.html
 / f_error  - Test for an error on a file                http://elm-chan.org/fsw/ff/en/error.html
 / f_printf - Write a formatted string                   http://elm-chan.org/fsw/ff/en/printf.html
 /               f_printf(&fil, "%d", 1234);             "1234"
 /               f_printf(&fil, "%6d,%3d%%", -200, 5);   "  -200,  5%"
 /               f_printf(&fil, "%ld", 12345L);          "12345"
 /               f_printf(&fil, "%06d", 25);             "000025"
 /               f_printf(&fil, "%06d", -25);            "000-25"
 /               f_printf(&fil, "%-6d", 25);             "25    "
 /               f_printf(&fil, "%u", -1);               "65535" or "4294967295"
 /               f_printf(&fil, "%04x", 0xAB3);          "0ab3"
 /               f_printf(&fil, "%08LX", 0x123ABCL);     "00123ABC"
 /               f_printf(&fil, "%016b", 0x550F);        "0101010100001111"
 /               f_printf(&fil, "%s", "String");         "String"
 /               f_printf(&fil, "%8s", "abc");           "     abc"
 /               f_printf(&fil, "%-8s", "abc");          "abc     "
 /               f_printf(&fil, "%c", 'a');              "a"
 /               f_printf(&fil, "%f", 10.0);             f_printf lacks floating point support (TIP)
 /                                                                                             =====
 / (TIP)     f_printf(&fil, "%s", ftostr(10.0, 7, 2));   "  10.00" (total 7 positions including 2 decimal places) 
   =====                          ==================
inline char * ftostr(float f, uint8_t len, uint8_t dec) {  // float to formatted char str
  char b[++len];
  dtostrf(f, len, dec, b);
  return b;
}

 /--------------------------------------------------------------------------------------------/
 / * Append to file? Use f_lseek and f_size
 /  char fileName[] = "write.txt", txtToWrite[] = "This text is added to the file";
 /  uint16_t b;  // bytes written
 /  // Opens the file if it is existing.If not, a new file is created
 /  fr = f_open(&fil, fileName, FA_OPEN_ALWAYS);
 /  fr = f_lseek(&fil, f_size(&fil));  // Move read/write pointer to end of file
 /  // append to file, strip null character ('\0') 
 /  fr = f_write(&fil, txtToWrite, sizeoff(txtToWrite) - 1, &b);
 /--------------------------------------------------------------------------------------------

ERRORS
======
 /  FR_OK = 0,                (0) Succeeded 
 /  FR_DISK_ERR,              (1) A hard error occurred in the low level disk I/O layer 
 /  FR_INT_ERR,               (2) Assertion failed 
 /  FR_NOT_READY,             (3) The physical drive cannot work 
 /  FR_NO_FILE,               (4) Could not find the file 
 /  FR_NO_PATH,               (5) Could not find the path 
 /  FR_INVALID_NAME,          (6) The path name format is invalid 
 /  FR_DENIED,                (7) Access denied due to prohibited access or directory full 
 /  FR_EXIST,                 (8) Access denied due to prohibited access 
 /  FR_INVALID_OBJECT,        (9) The file/directory object is invalid 
 /  FR_WRITE_PROTECTED,      (10) The physical drive is write protected 
 /  FR_INVALID_DRIVE,        (11) The logical drive number is invalid 
 /  FR_NOT_ENABLED,          (12) The volume has no work area 
 /  FR_NO_FILESYSTEM,        (13) There is no valid FAT volume 
 /  FR_MKFS_ABORTED,         (14) The f_mkfs() aborted due to any parameter error 
 /  FR_TIMEOUT,              (15) Could not get a grant to access the volume within defined period 
 /  FR_LOCKED,               (16) The operation is rejected according to the file sharing policy 
 /  FR_NOT_ENOUGH_CORE,      (17) LFN working buffer could not be allocated 
 /  FR_TOO_MANY_OPEN_FILES,  (18) Number of open files > _FS_SHARE 
 /  FR_INVALID_PARAMETER     (19) Given parameter is invalid 
*/